library angular2.src.compiler.api;

import "package:angular2/src/core/facade/lang.dart"
    show isPresent, normalizeBool;
import "html_ast.dart" show HtmlAst;
import "package:angular2/src/core/change_detection/change_detection.dart"
    show ChangeDetectionStrategy;

class TypeMetadata {
  dynamic type;
  String typeName;
  String typeUrl;
  TypeMetadata({type, typeName, typeUrl}) {
    this.type = type;
    this.typeName = typeName;
    this.typeUrl = typeUrl;
  }
}

class ChangeDetectionMetadata {
  ChangeDetectionStrategy changeDetection;
  List<String> properties;
  List<String> events;
  Map<String, String> hostListeners;
  Map<String, String> hostProperties;
  ChangeDetectionMetadata(
      {changeDetection, properties, events, hostListeners, hostProperties}) {
    this.changeDetection = changeDetection;
    this.properties = properties;
    this.events = events;
    this.hostListeners = hostListeners;
    this.hostProperties = hostProperties;
  }
}

class TemplateMetadata {
  ViewEncapsulation encapsulation;
  List<HtmlAst> nodes;
  List<String> styles;
  List<String> styleAbsUrls;
  List<String> ngContentSelectors;
  TemplateMetadata(
      {encapsulation, nodes, styles, styleAbsUrls, ngContentSelectors}) {
    this.encapsulation = encapsulation;
    this.nodes = nodes;
    this.styles = styles;
    this.styleAbsUrls = styleAbsUrls;
    this.ngContentSelectors = ngContentSelectors;
  }
}

/**
 * How the template and styles of a view should be encapsulated.
 */
enum ViewEncapsulation {
  /**
   * Emulate scoping of styles by preprocessing the style rules
   * and adding additional attributes to elements. This is the default.
   */
  Emulated,
  /**
   * Uses the native mechanism of the renderer. For the DOM this means creating a ShadowRoot.
   */
  Native,
  /**
   * Don't scope the template nor the styles.
   */
  None
}

class DirectiveMetadata {
  TypeMetadata type;
  bool isComponent;
  String selector;
  Map<String, String> hostAttributes;
  ChangeDetectionMetadata changeDetection;
  TemplateMetadata template;
  DirectiveMetadata(
      {type,
      isComponent,
      selector,
      hostAttributes,
      changeDetection,
      template}) {
    this.type = type;
    this.isComponent = normalizeBool(isComponent);
    this.selector = selector;
    this.hostAttributes = hostAttributes;
    this.changeDetection = changeDetection;
    this.template = template;
  }
}
