library angular2.src.compiler.html_ast;

import "package:angular2/src/core/facade/lang.dart" show isPresent;

abstract class HtmlAst {
  String sourceInfo;
  dynamic visit(HtmlAstVisitor visitor);
}

class HtmlTextAst implements HtmlAst {
  String value;
  String sourceInfo;
  HtmlTextAst(this.value, this.sourceInfo) {}
  dynamic visit(HtmlAstVisitor visitor) {
    return visitor.visitText(this);
  }
}

class HtmlAttrAst implements HtmlAst {
  String name;
  String value;
  String sourceInfo;
  HtmlAttrAst(this.name, this.value, this.sourceInfo) {}
  dynamic visit(HtmlAstVisitor visitor) {
    return visitor.visitAttr(this);
  }
}

class HtmlElementAst implements HtmlAst {
  String name;
  List<HtmlAttrAst> attrs;
  List<HtmlAst> children;
  String sourceInfo;
  HtmlElementAst(this.name, this.attrs, this.children, this.sourceInfo) {}
  dynamic visit(HtmlAstVisitor visitor) {
    return visitor.visitElement(this);
  }
}

abstract class HtmlAstVisitor {
  dynamic visitElement(HtmlElementAst ast);
  dynamic visitAttr(HtmlAttrAst ast);
  dynamic visitText(HtmlTextAst ast);
}

List<dynamic> htmlVisitAll(HtmlAstVisitor visitor, List<HtmlAst> asts) {
  var result = [];
  asts.forEach((ast) {
    var astResult = ast.visit(visitor);
    if (isPresent(astResult)) {
      result.add(astResult);
    }
  });
  return result;
}
