// Some of the code comes from WebComponents.JS

// https://github.com/webcomponents/webcomponentsjs/blob/master/src/HTMLImports/path.js
library angular2.src.compiler.style_url_resolver;

import "package:angular2/di.dart" show Injectable;
import "package:angular2/src/core/facade/lang.dart"
    show RegExp, RegExpWrapper, StringWrapper;
import "package:angular2/src/core/services/url_resolver.dart" show UrlResolver;

/**
 * Rewrites URLs by resolving '@import' and 'url()' URLs from the given base URL,
 * removes and returns the @import urls
 */
@Injectable()
class StyleUrlResolver {
  UrlResolver _resolver;
  StyleUrlResolver(this._resolver) {}
  String resolveUrls(String cssText, String baseUrl) {
    cssText = this._replaceUrls(cssText, _cssUrlRe, baseUrl);
    return cssText;
  }

  StyleWithImports extractImports(String cssText) {
    var foundUrls = [];
    cssText = this._extractUrls(cssText, _cssImportRe, foundUrls);
    return new StyleWithImports(cssText, foundUrls);
  }

  _replaceUrls(String cssText, RegExp re, String baseUrl) {
    return StringWrapper.replaceAllMapped(cssText, re, (m) {
      var pre = m[1];
      var originalUrl = m[2];
      if (RegExpWrapper.test(_dataUrlRe, originalUrl)) {
        // Do not attempt to resolve data: URLs
        return m[0];
      }
      var url = StringWrapper.replaceAll(originalUrl, _quoteRe, "");
      var post = m[3];
      var resolvedUrl = this._resolver.resolve(baseUrl, url);
      return pre + "'" + resolvedUrl + "'" + post;
    });
  }

  _extractUrls(String cssText, RegExp re, List<String> foundUrls) {
    return StringWrapper.replaceAllMapped(cssText, re, (m) {
      var originalUrl = m[2];
      if (RegExpWrapper.test(_dataUrlRe, originalUrl)) {
        // Do not attempt to resolve data: URLs
        return m[0];
      }
      var url = StringWrapper.replaceAll(originalUrl, _quoteRe, "");
      foundUrls.add(url);
      return "";
    });
  }
}

class StyleWithImports {
  String style;
  List<String> styleUrls;
  StyleWithImports(this.style, this.styleUrls) {}
}

var _cssUrlRe = new RegExp(r'(url\()([^)]*)(\))');
var _cssImportRe = new RegExp(r'(@import[\s]+(?:url\()?)[' +
    "'" +
    r'"]?([^' +
    "'" +
    r'"\)]*)[' +
    "'" +
    r'"]?(.*;)');
var _quoteRe = new RegExp(r'[' + "'" + r'"]');
var _dataUrlRe = new RegExp(r'^[' + "'" + r'"]?data:');
