library angular2.src.core.application_ref;

import "package:angular2/src/core/compiler/dynamic_component_loader.dart"
    show ComponentRef;
import "package:angular2/di.dart" show Injector;
import "package:angular2/src/core/facade/lang.dart" show Type;

/**
 * Represents a Angular's representation of an Application.
 *
 * `ApplicationRef` represents a running application instance. Use it to retrieve the host
 * component, injector,
 * or dispose of an application.
 */
class ApplicationRef {
  ComponentRef _hostComponent;
  Injector _injector;
  Type _hostComponentType;
  /**
   * @private
   */
  ApplicationRef(
      ComponentRef hostComponent, Type hostComponentType, Injector injector) {
    this._hostComponent = hostComponent;
    this._injector = injector;
    this._hostComponentType = hostComponentType;
  }
  /**
   * Returns the current [ComponentMetadata] type.
   */
  Type get hostComponentType {
    return this._hostComponentType;
  }

  /**
   * Returns the current [ComponentMetadata] instance.
   */
  dynamic get hostComponent {
    return this._hostComponent.instance;
  }

  /**
   * Dispose (un-load) the application.
   */
  void dispose() {
    // TODO: We also need to clean up the Zone, ... here!
    this._hostComponent.dispose();
  }

  /**
   * Returns the root application [Injector].
   */
  Injector get injector {
    return this._injector;
  }
}
