library angular2.src.core.change_detection.exceptions;

import "package:angular2/src/core/facade/lang.dart" show BaseException;

/**
 * An error thrown if application changes model breaking the top-down data flow.
 *
 * Angular expects that the data flows from top (root) component to child (leaf) components.
 * This is known as directed acyclic graph. This allows Angular to only execute change detection
 * once and prevents loops in change detection data flow.
 *
 * This exception is only thrown in dev mode.
 */
class ExpressionChangedAfterItHasBeenCheckedException extends BaseException {
  ExpressionChangedAfterItHasBeenCheckedException(
      String exp, dynamic oldValue, dynamic currValue, dynamic context)
      : super('''Expression \'${ exp}\' has changed after it was checked. ''' +
            '''Previous value: \'${ oldValue}\'. Current value: \'${ currValue}\'''') {
    /* super call moved to initializer */;
  }
}

/**
 * Thrown when an expression evaluation raises an exception.
 *
 * This error wraps the original exception, this is done to attach expression location information.
 */
class ChangeDetectionError extends BaseException {
  /**
   * Location of the expression.
   */
  String location;
  ChangeDetectionError(String exp, dynamic originalException,
      dynamic originalStack, dynamic context)
      : super('''${ originalException} in [${ exp}]''', originalException,
            originalStack, context) {
    /* super call moved to initializer */;
    this.location = exp;
  }
}

/**
 * Thrown when change detector executes on dehydrated view.
 *
 * This is angular internal error.
 */
class DehydratedException extends BaseException {
  DehydratedException()
      : super("Attempt to detect changes on a dehydrated detector.") {
    /* super call moved to initializer */;
  }
}
