library angular2.src.core.change_detection.interfaces;

import "parser/locals.dart" show Locals;
import "binding_record.dart" show BindingTarget, BindingRecord;
import "directive_record.dart" show DirectiveIndex, DirectiveRecord;
import "constants.dart" show ChangeDetectionStrategy;
import "change_detector_ref.dart" show ChangeDetectorRef;

/**
 * Interface used by Angular to control the change detection strategy for an application.
 *
 * Angular implements the following change detection strategies by default:
 *
 * - [DynamicChangeDetection]: slower, but does not require `eval()`.
 * - [JitChangeDetection]: faster, but requires `eval()`.
 *
 * In JavaScript, you should always use `JitChangeDetection`, unless you are in an environment that
 *has
 * [CSP](https://developer.mozilla.org/en-US/docs/Web/Security/CSP), such as a Chrome Extension.
 *
 * In Dart, use `DynamicChangeDetection` during development. The Angular transformer generates an
 *analog to the
 * `JitChangeDetection` strategy at compile time.
 *
 *
 * See: [DynamicChangeDetection], [JitChangeDetection],
 * [PreGeneratedChangeDetection]
 *
 * # Example
 * ```javascript
 * bootstrap(MyApp, [bind(ChangeDetection).toValue(new DynamicChangeDetection())]);
 * ```
 */
class ChangeDetection {
  ProtoChangeDetector getProtoChangeDetector(
      String id, ChangeDetectorDefinition definition) {
    return null;
  }

  bool get generateDetectors {
    return null;
  }

  ChangeDetectorGenConfig get genConfig {
    return null;
  }

  const ChangeDetection();
}

class DebugContext {
  dynamic element;
  dynamic componentElement;
  dynamic directive;
  dynamic context;
  dynamic locals;
  dynamic injector;
  DebugContext(this.element, this.componentElement, this.directive,
      this.context, this.locals, this.injector) {}
}

abstract class ChangeDispatcher {
  DebugContext getDebugContext(num elementIndex, DirectiveIndex directiveIndex);
  void notifyOnBinding(BindingTarget bindingTarget, dynamic value);
  void logBindingUpdate(BindingTarget bindingTarget, dynamic value);
  void notifyAfterContentChecked();
  void notifyAfterViewChecked();
}

abstract class ChangeDetector {
  ChangeDetector parent;
  ChangeDetectionStrategy mode;
  ChangeDetectorRef ref;
  void addChild(ChangeDetector cd);
  void addShadowDomChild(ChangeDetector cd);
  void removeChild(ChangeDetector cd);
  void removeShadowDomChild(ChangeDetector cd);
  void remove();
  void hydrate(
      dynamic context, Locals locals, dynamic directives, dynamic pipes);
  void dehydrate();
  void markPathToRootAsCheckOnce();
  handleEvent(String eventName, num elIndex, Locals locals);
  void detectChanges();
  void checkNoChanges();
}

abstract class ProtoChangeDetector {
  ChangeDetector instantiate(ChangeDispatcher dispatcher);
}

class ChangeDetectorGenConfig {
  bool genCheckNoChanges;
  bool genDebugInfo;
  bool logBindingUpdate;
  ChangeDetectorGenConfig(
      this.genCheckNoChanges, this.genDebugInfo, this.logBindingUpdate) {}
}

class ChangeDetectorDefinition {
  String id;
  ChangeDetectionStrategy strategy;
  List<String> variableNames;
  List<BindingRecord> bindingRecords;
  List<BindingRecord> eventRecords;
  List<DirectiveRecord> directiveRecords;
  ChangeDetectorGenConfig genConfig;
  ChangeDetectorDefinition(
      this.id,
      this.strategy,
      this.variableNames,
      this.bindingRecords,
      this.eventRecords,
      this.directiveRecords,
      this.genConfig) {}
}
