library angular2.src.core.render.dom.events.event_manager;

import "package:angular2/src/core/facade/lang.dart"
    show isBlank, BaseException, isPresent, StringWrapper;
import "package:angular2/src/core/dom/dom_adapter.dart" show DOM;
import "package:angular2/src/core/zone/ng_zone.dart" show NgZone;

class EventManager {
  List<EventManagerPlugin> _plugins;
  NgZone _zone;
  EventManager(this._plugins, this._zone) {
    for (var i = 0; i < _plugins.length; i++) {
      _plugins[i].manager = this;
    }
  }
  addEventListener(dynamic element, String eventName, Function handler) {
    var plugin = this._findPluginFor(eventName);
    plugin.addEventListener(element, eventName, handler);
  }

  Function addGlobalEventListener(
      String target, String eventName, Function handler) {
    var plugin = this._findPluginFor(eventName);
    return plugin.addGlobalEventListener(target, eventName, handler);
  }

  NgZone getZone() {
    return this._zone;
  }

  EventManagerPlugin _findPluginFor(String eventName) {
    var plugins = this._plugins;
    for (var i = 0; i < plugins.length; i++) {
      var plugin = plugins[i];
      if (plugin.supports(eventName)) {
        return plugin;
      }
    }
    throw new BaseException(
        '''No event manager plugin found for event ${ eventName}''');
  }
}

class EventManagerPlugin {
  EventManager manager;
  // That is equivalent to having supporting $event.target
  bool supports(String eventName) {
    return false;
  }

  addEventListener(dynamic element, String eventName, Function handler) {
    throw "not implemented";
  }

  Function addGlobalEventListener(
      String element, String eventName, Function handler) {
    throw "not implemented";
  }
}

class DomEventsPlugin extends EventManagerPlugin {
  EventManager manager;
  // This plugin should come last in the list of plugins, because it accepts all

  // events.
  bool supports(String eventName) {
    return true;
  }

  addEventListener(dynamic element, String eventName, Function handler) {
    var zone = this.manager._zone;
    var outsideHandler = (event) => zone.run(() => handler(event));
    this.manager._zone.runOutsideAngular(() {
      DOM.on(element, eventName, outsideHandler);
    });
  }

  Function addGlobalEventListener(
      String target, String eventName, Function handler) {
    var element = DOM.getGlobalEventTarget(target);
    var zone = this.manager._zone;
    var outsideHandler = (event) => zone.run(() => handler(event));
    return this.manager._zone.runOutsideAngular(() {
      return DOM.onAndCancel(element, eventName, outsideHandler);
    });
  }
}
