library angular2.src.web_workers.shared.message_bus;

import "package:angular2/src/core/facade/async.dart" show EventEmitter;
import "package:angular2/src/core/facade/lang.dart" show BaseException;
export "package:angular2/src/core/facade/async.dart" show EventEmitter, Stream;

_abstract() {
  throw new BaseException("This method is abstract");
}

/**
 * Message Bus is a low level API used to communicate between the UI and the background.
 * Communication is based on a channel abstraction. Messages published in a
 * given channel to one MessageBusSink are received on the same channel
 * by the corresponding MessageBusSource.
 */
class MessageBus implements MessageBusSource, MessageBusSink {
  /**
   * Returns an [EventEmitter] that emits every time a messsage
   * is received on the given channel.
   */
  EventEmitter from(String channel) {
    throw _abstract();
  }

  /**
   * Returns an [EventEmitter] for the given channel
   * To publish methods to that channel just call next (or add in dart) on the returned emitter
   */
  EventEmitter to(String channel) {
    throw _abstract();
  }
}

abstract class MessageBusSource {
  /**
   * Returns an [EventEmitter] that emits every time a messsage
   * is received on the given channel.
   */
  EventEmitter from(String channel);
}

abstract class MessageBusSink {
  /**
   * Returns an [EventEmitter] for the given channel
   * To publish methods to that channel just call next (or add in dart) on the returned emitter
   */
  EventEmitter to(String channel);
}
