library angular2.test.core.directive_lifecycle_integration_spec;

import "package:angular2/test_lib.dart"
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        describe,
        expect,
        iit,
        inject,
        it,
        xdescribe,
        xit,
        Log,
        TestComponentBuilder;
import "package:angular2/metadata.dart"
    show Directive, Component, View, ViewMetadata, LifecycleEvent;

main() {
  describe("directive lifecycle integration spec", () {
    it(
        "should invoke lifecycle methods onChanges > onInit > doCheck > afterContentChecked",
        inject([TestComponentBuilder, Log, AsyncTestCompleter],
            (TestComponentBuilder tcb, Log log, async) {
          tcb
              .overrideView(
                  MyComp,
                  new ViewMetadata(
                      template: "<div [field]=\"123\" lifecycle></div>",
                      directives: [LifecycleCmp]))
              .createAsync(MyComp)
              .then((tc) {
            tc.detectChanges();
            expect(log.result()).toEqual(
                "onChanges; onInit; doCheck; afterContentInit; afterContentChecked; child_doCheck; " +
                    "afterViewInit; afterViewChecked");
            log.clear();
            tc.detectChanges();
            expect(log.result()).toEqual(
                "doCheck; afterContentChecked; child_doCheck; afterViewChecked");
            async.done();
          });
        }));
  });
}

@Directive(
    selector: "[lifecycle-dir]", lifecycle: const [LifecycleEvent.DoCheck])
class LifecycleDir {
  Log log;
  LifecycleDir(this.log) {}
  doCheck() {
    this.log.add("child_doCheck");
  }
}

@Component(
    selector: "[lifecycle]",
    properties: const ["field"],
    lifecycle: const [
      LifecycleEvent.OnChanges,
      LifecycleEvent.OnInit,
      LifecycleEvent.DoCheck,
      LifecycleEvent.AfterContentInit,
      LifecycleEvent.AfterContentChecked,
      LifecycleEvent.AfterViewInit,
      LifecycleEvent.AfterViewChecked
    ])
@View(
    template: '''<div lifecycle-dir></div>''', directives: const [LifecycleDir])
class LifecycleCmp {
  Log log;
  var field;
  LifecycleCmp(this.log) {}
  onChanges(_) {
    this.log.add("onChanges");
  }

  onInit() {
    this.log.add("onInit");
  }

  doCheck() {
    this.log.add("doCheck");
  }

  afterContentInit() {
    this.log.add("afterContentInit");
  }

  afterContentChecked() {
    this.log.add("afterContentChecked");
  }

  afterViewInit() {
    this.log.add("afterViewInit");
  }

  afterViewChecked() {
    this.log.add("afterViewChecked");
  }
}

@Component(selector: "my-comp")
@View(directives: const [])
class MyComp {}
