library angular2.test.router.router_link_spec;

import "package:angular2/test_lib.dart"
    show
        AsyncTestCompleter,
        beforeEach,
        ddescribe,
        xdescribe,
        describe,
        dispatchEvent,
        expect,
        iit,
        inject,
        beforeEachBindings,
        it,
        xit,
        TestComponentBuilder,
        By;
import "spies.dart" show SpyRouter, SpyLocation;
import "package:angular2/angular2.dart" show bind, Component, View;
import "package:angular2/router.dart"
    show
        Location,
        Router,
        RootRouter,
        RouteRegistry,
        Pipeline,
        RouterLink,
        RouterOutlet,
        Route,
        RouteParams;
import "package:angular2/src/router/instruction.dart"
    show Instruction, ComponentInstruction;
import "package:angular2/src/core/dom/dom_adapter.dart" show DOM;

var dummyInstruction =
    new Instruction(new ComponentInstruction("detail", [], null), null, {});
main() {
  describe("router-link directive", () {
    TestComponentBuilder tcb;
    beforeEachBindings(() => [
          bind(Location).toValue(makeDummyLocation()),
          bind(Router).toValue(makeDummyRouter())
        ]);
    beforeEach(inject([TestComponentBuilder], (tcBuilder) {
      tcb = tcBuilder;
    }));
    it(
        "should update a[href] attribute",
        inject([AsyncTestCompleter], (async) {
          tcb.createAsync(TestComponent).then((testComponent) {
            testComponent.detectChanges();
            var anchorElement = testComponent.query(By.css("a")).nativeElement;
            expect(DOM.getAttribute(anchorElement, "href")).toEqual("/detail");
            async.done();
          });
        }));
    it(
        "should call router.navigate when a link is clicked",
        inject([AsyncTestCompleter, Router], (async, router) {
          tcb.createAsync(TestComponent).then((testComponent) {
            testComponent.detectChanges();
            // TODO: shouldn't this be just 'click' rather than '^click'?
            testComponent.query(By.css("a")).triggerEventHandler("click", null);
            expect(router.spy("navigateInstruction"))
                .toHaveBeenCalledWith(dummyInstruction);
            async.done();
          });
        }));
  });
}

@Component(selector: "my-comp")
class MyComp {
  var name;
}

@Component(selector: "user-cmp")
@View(template: "hello {{user}}")
class UserCmp {
  String user;
  UserCmp(RouteParams params) {
    this.user = params.get("name");
  }
}

@Component(selector: "test-component")
@View(
    template: '''
    <div>
      <a [router-link]="[\'/detail\']">detail view</a>
    </div>''',
    directives: const [RouterLink])
class TestComponent {}

makeDummyLocation() {
  var dl = new SpyLocation();
  dl.spy("normalizeAbsolutely").andCallFake((url) => url);
  return dl;
}

makeDummyRouter() {
  var dr = new SpyRouter();
  dr.spy("generate").andCallFake((routeParams) => dummyInstruction);
  dr.spy("isRouteActive").andCallFake((_) => false);
  dr.spy("navigateInstruction");
  return dr;
}
