library angular2.test.web_workers.shared.web_worker_test_util;

import "package:angular2/src/core/facade/collection.dart"
    show Map, StringMapWrapper, ListWrapper;
import "package:angular2/src/web_workers/shared/message_bus.dart"
    show MessageBusSink, MessageBusSource, MessageBus;
import "mock_event_emitter.dart" show MockEventEmitter;

/**
 * Returns two MessageBus instances that are attached to each other.
 * Such that whatever goes into one's sink comes out the others source.
 */
PairedMessageBuses createPairedMessageBuses() {
  Map<String, MockEventEmitter> firstChannels = {};
  var workerMessageBusSink = new MockMessageBusSink(firstChannels);
  var uiMessageBusSource = new MockMessageBusSource(firstChannels);
  Map<String, MockEventEmitter> secondChannels = {};
  var uiMessageBusSink = new MockMessageBusSink(secondChannels);
  var workerMessageBusSource = new MockMessageBusSource(secondChannels);
  return new PairedMessageBuses(
      new MockMessageBus(uiMessageBusSink, uiMessageBusSource),
      new MockMessageBus(workerMessageBusSink, workerMessageBusSource));
}

class PairedMessageBuses {
  MessageBus ui;
  MessageBus worker;
  PairedMessageBuses(this.ui, this.worker) {}
}

class MockMessageBusSource implements MessageBusSource {
  Map<String, MockEventEmitter> _channels;
  MockMessageBusSource(this._channels) {}
  MockEventEmitter from(String channel) {
    if (!StringMapWrapper.contains(this._channels, channel)) {
      this._channels[channel] = new MockEventEmitter();
    }
    return this._channels[channel];
  }
}

class MockMessageBusSink implements MessageBusSink {
  Map<String, MockEventEmitter> _channels;
  MockMessageBusSink(this._channels) {}
  MockEventEmitter to(String channel) {
    if (!StringMapWrapper.contains(this._channels, channel)) {
      this._channels[channel] = new MockEventEmitter();
    }
    return this._channels[channel];
  }
}

class MockMessageBus extends MessageBus {
  MockMessageBusSink sink;
  MockMessageBusSource source;
  MockMessageBus(this.sink, this.source) : super() {
    /* super call moved to initializer */;
  }
  MockEventEmitter to(String channel) {
    return this.sink.to(channel);
  }

  MockEventEmitter from(String channel) {
    return this.source.from(channel);
  }
}
